#!/usr/bin/python

"""
game.py - main in-game scene classes
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *
from math import *
import numpy as np

import resources
import model
import spacecraft
import visuals

from common import FadeInOut

import random

class ProgressData(object):
    def __init__(self):
        self.current_level = "level_"
    
    def LoadProgressData(self):
        f = open(self.savepath, "r");
        self.current_level = f.readline().split()[0]
        f.close()
    
    def SaveProgressData(self):
        f = open(self.savepath, "w");
        f.write('%s\n'%(self.current_level))
        f.close()
    
    def Reset(self):
        self.current_level = "level_"

class MissionTracking(object):
    def __init__(self,name,parent):
        self.name = name
        self.parent = parent
        self.current_music = resources.all_mission_data[self.name]['music']
        self.player_start_pos = resources.all_mission_data[self.name]['player']
        self.jump_pos = resources.all_mission_data[self.name]['jump']
        
        self.current_objective = -1
        self.finished_obj = True
        self.get_objship_list()
        
        self.current_voice = None
        self.voice_to = 0
        
        self.train_voice_to = 0
        self.train_current_voice = 0
    
    def get_objship_list(self):
        self.objship_list = []
        objective_data = resources.all_mission_data[self.name]['obj'][self.current_objective]
        if objective_data[0] == 'ship':
            for ind in objective_data[1]:
                self.objship_list.append(resources.all_mission_data[self.name]['ships'][ind][2])
    
    def GetObjLoc(self):
        objective_data = resources.all_mission_data[self.name]['obj'][self.current_objective]
        if objective_data[0] == 'loc':
            self.obj_loc = objective_data[1]
        elif objective_data[0] == 'jump':
            self.obj_loc = self.parent.jumpgates[0].pos
        elif objective_data[0] == 'ship':
            self.obj_loc = ['ships']
            for i in xrange(len(self.parent.spacecraft)):
                if self.parent.spacecraft[i].id in self.objship_list:
                    self.obj_loc.append(i)
    
    def obj_completed(self):
        mdata = resources.all_mission_data[self.name]['obj'][self.current_objective]
        if len(mdata) == 4: # completion voice data
            self.current_voice = mdata[3]
            self.voice_to = 60
        self.load_objective(self.current_objective+1)
    
    def load_objective(self, obj):
        self.current_objective = obj
        if self.current_objective < len(resources.all_mission_data[self.name]['obj']):
            if resources.all_mission_data[self.name]['obj'][self.current_objective][0] == 'jump':
                self.parent.jumpgates[0].active = True
            self.get_objship_list()
            self.GetObjLoc()
            if resources.CheckVoice():
                resources.StopVoice()
            resources.PlayVoice('new_obj')
        else:
            self.finished_obj = True
    
    def Update(self):
        if not self.finished_obj:
            objective_data = resources.all_mission_data[self.name]['obj'][self.current_objective]
            if objective_data[0] == 'loc':
                rel_pos = self.parent.camera.pos-objective_data[1]
                dist = np.linalg.norm(rel_pos)
                if dist < 10.0:
                    self.obj_completed()
            elif objective_data[0] == 'jump':
                if self.parent.warping_out:
                    self.obj_completed()
            elif objective_data[0] == 'ship':
                left_alive = False
                for i in xrange(len(self.parent.spacecraft)):
                    if self.parent.spacecraft[i].id in self.objship_list:
                        if self.parent.spacecraft[i].alive:
                            left_alive = True
                if not left_alive:
                    self.obj_completed()
        if self.train_voice_to > 0 and not resources.CheckVoice():
            self.train_voice_to -= 1
            if self.train_voice_to == 0:
                self.train_current_voice += 1
                if self.train_current_voice == 3:
                    self.train_voice_to = 0
                    self.finished_obj = False
                    self.load_objective(0)
                    if not resources.all_mission_data[self.parent.mission_name]['init_voice'] == None:
                        resources.PlayVoice(resources.all_mission_data[self.parent.mission_name]['init_voice'])
                else:
                    resources.PlayVoice(resources.training_voicelist[self.train_current_voice])
                    self.train_voice_to = 60
                    
        if self.voice_to > 0:
            self.voice_to -= 1
            if self.voice_to == 0:
                resources.PlayVoice(self.current_voice)

class MainGame(GameScene):
    def __init__(self, director, window_size):
        super(MainGame, self).__init__(director)
        self.window_size = window_size
        
        # frame rate recording
        self.avgframerate = -1
        self.frsamples = 0
        
        # Music
        self.current_music = 'none'
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
        
        self.backgroundfade = pygame.Surface(window_size)
        self.backgroundfade.fill((0,0,0))
        self.backgroundfade.convert()
        
        self.backgroundwarp = pygame.Surface(window_size)
        self.backgroundwarp.fill((255,255,255))
        self.backgroundwarp.convert()
        
        # Options data
        self.mouse_sensitivity = 0.005
        
        # Initialise player data
        self.progress_data = ProgressData()
        
        self.fade = FadeInOut(30)
    
    def on_switchto(self, switchtoargs):
        
        lvlreset = switchtoargs[0]
        if not lvlreset:
            return
        self.mission_name = switchtoargs[1]
        
        pygame.mouse.set_pos([320,240])
        self.mp = pygame.mouse.get_pos()
        
        # Save progress
        self.progress_data.current_level = self.mission_name
        self.progress_data.SaveProgressData()
        
        # Load mission objective data
        self.mission = MissionTracking(self.mission_name,self)
        if self.mission_name == 'training':
            self.mission.finished_obj = True
            self.mission.train_voice_to = 60
            resources.PlayVoice(resources.training_voicelist[self.mission.train_current_voice])
        #else:
        #    self.mission.finished_obj = False
        #    self.mission.load_objective(0)
        #    if not resources.all_mission_data[self.mission_name]['init_voice'] == None:
        #        resources.PlayVoice(resources.all_mission_data[self.mission_name]['init_voice'])
        
        # Initialise Camera
        self.camera = model.Camera(self.window_size,self.mission.player_start_pos)
        self.player = spacecraft.Player(self,self.camera)
        self.camera.player = self.player
        self.hud = spacecraft.HUD(self,self.window_size,self.player)
        
        # Initialise background visuals
        if self.mission_name == 'mission6':
            self.sky = visuals.Sky(self.camera,type='stars002')
        else:
            self.sky = visuals.Sky(self.camera,type='stars001')
        self.motiondust = visuals.MotionDust(self,self.camera)
        
        # Initialise motion control
        self.motion = spacecraft.MotionControl(self,resources.controlmap,self.camera,self.player)
        mp = pygame.mouse.get_pos()
        self.motion.mouse_rel_ignore = mp
        
        # Initialise Spacecraft
        self.spacecraft = []
        for i in xrange(len(resources.all_mission_data[self.mission_name]['ships'])):
            type = resources.all_mission_data[self.mission_name]['ships'][i][0]
            loc = resources.all_mission_data[self.mission_name]['ships'][i][1]
            id = resources.all_mission_data[self.mission_name]['ships'][i][2]
            self.spacecraft.append(spacecraft.Spacecraft(self,self.camera,loc,type,id=id))
        
        for i in xrange(len(self.spacecraft)):
            if self.spacecraft[i].label == 'hostile':
                self.hud.enemylist.append(self.spacecraft[i])
        
        # Initialise Jump Gates
        self.jumpgates = []
        self.jumpgates.append(spacecraft.JumpGate(self,self.camera,self.mission.jump_pos,active=False))
        
        if self.mission_name == 'mission6':
            self.jumpgates[0].active = True
        
        # Initialise lighting sources
        self.lights = [resources.light_main,resources.light_main2]
        #self.lights = [resources.light_main]
        for l in self.lights:
            self.camera.AddLight(l)
        
        # projectiles
        self.explosioneffects = visuals.ExplosionEffects(self,self.camera)
        self.debris = spacecraft.Debris(self.camera)
        self.projectiles = spacecraft.Projectiles(self.camera)
        self.projectiles.spacecraft = self.spacecraft
        self.projectiles.explosioneffects = self.explosioneffects
        
        self.camera.projectiles = self.projectiles
        self.camera.motiondust = self.motiondust
        self.camera.player = self.player
        
        # start music
        if not self.current_music == self.mission.current_music:
            self.current_music = self.mission.current_music
            pygame.mixer.music.stop()
            if not self.current_music == 'none':
                pygame.mixer.music.load(resources.musicpaths[self.current_music])
                pygame.mixer.music.set_volume(0.5)
                pygame.mixer.music.play(-1)
        
        self.fade.Reset()
        self.fade.FadeIn()
        self.leaving_mission = False
        self.warping_out = False
        self.warp_to = 0
        
        if not self.mission_name == 'training':
            self.mission.finished_obj = False
            self.mission.load_objective(0)
            if not resources.all_mission_data[self.mission_name]['init_voice'] == None:
                resources.PlayVoice(resources.all_mission_data[self.mission_name]['init_voice'])
        
        self.mission.GetObjLoc()
        
    def on_update(self):
        
        self.fade.Update()
        if self.leaving_mission and self.fade.direction == 'in':
            if self.player.alive:
                musicfade = True
            else:
                musicfade = False
            self.fade.FadeOut(musicfade)
        if self.fade.finished_out:
            if self.player.alive:
                self.current_music = 'none'
                next_mission = resources.all_mission_data[self.mission_name]['next_mission']
                if type(next_mission) == type(''):
                    self.director.change_scene('maingame', [True,next_mission])
                else:
                    self.director.change_scene(next_mission[0], next_mission[1])
            else:
                self.director.change_scene('maingame', [True,self.mission_name])
        
        # Update motion control
        self.motion.UpdateMotion()
        self.player.Update()
        
        # Other game objects
        self.projectiles.Update()
        for s in self.spacecraft:
            s.Update()
        for j in self.jumpgates:
            j.Update()
        self.explosioneffects.Update()
        self.debris.Update()
        self.motiondust.Update()
        
        if self.warp_to > 0:
            self.warp_to -= 1
            if self.warp_to == 0:
                self.leaving_mission = True
        
        # look for mission updates
        self.mission.Update()
        
        # framerate tracking
        self.frsamples += 1
        if self.frsamples == 1:
            self.avgframerate = self.director.framerate
        else:
            self.avgframerate = self.avgframerate + (self.director.framerate - self.avgframerate)/(self.frsamples)
        
    """
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                #self.director.change_scene(None, [])
                self.draw(self.h_pausescene.background)
                resources.PauseVoice()
                self.director.change_scene('pausegame', self.mission_name)
            if event.type == KEYDOWN or event.type == KEYUP or event.type == MOUSEBUTTONDOWN:
                self.motion.ProcessKeyEvent(event)
            if event.type == MOUSEMOTION:
                self.motion.ProcessMouseMotion(event)
        mp = pygame.mouse.get_pos()
        pygame.mouse.set_pos([320,240])
        self.motion.mouse_rel_ignore = ((320-mp[0]),(240-mp[1]))
    """
    
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                #self.director.change_scene(None, [])
                self.draw(self.h_pausescene.background)
                resources.PauseVoice()
                self.director.change_scene('pausegame', self.mission_name)
            if event.type == KEYDOWN or event.type == KEYUP or event.type == MOUSEBUTTONDOWN:
                self.motion.ProcessKeyEvent(event)
            #if event.type == MOUSEMOTION:
            #    self.motion.ProcessMouseMotion(event)
        self.mp = pygame.mouse.get_pos()
        pygame.mouse.set_pos([320,240])
    
    def draw(self, screen):
        screen.blit(self.background, (0, 0))
        self.sky.Draw(screen)
        self.camera.DrawScene(screen)
        #self.projectiles.Draw(screen)
        self.explosioneffects.Draw(screen)
        self.hud.Draw(screen)
    
    def on_draw(self, screen):
        self.draw(screen)
        if self.warping_out:
            if self.warp_to < 60:
                alpha = 255-int((255/60.0)*self.warp_to)
                self.backgroundwarp.set_alpha(alpha)
                screen.blit(self.backgroundwarp, (0, 0))
        self.backgroundfade.set_alpha(self.fade.alpha)
        screen.blit(self.backgroundfade, (0, 0))
