#!/usr/bin/python

"""
resources.py - Load all resource data once 
"""

import os, sys
import pygame
from pygame.locals import *
import numpy as np

import model

from textrect import *

class Model(object):
    def __init__(self):
        self.vertex = []
        self.face = []
    
    def Load(self, filepath):
        self.vertex = []
        self.face = []
        self.colour = []
        f = open(filepath, "r")
        while True:
            d = f.readline().split()
            if d[0] == 'element' and d[1] == 'vertex':
                n_vertex = int(d[2])
            elif d[0] == 'element' and d[1] == 'face':
                n_face = int(d[2])
            elif d[0] == 'end_header':
                break
        for i in xrange(n_vertex):
            self.vertex.append([float(i) for i in f.readline().split()])
        for i in xrange(n_face):
            self.face.append([int(i) for i in f.readline().split()][1:])
            colour_int = [int(i) for i in f.readline().split()]
            self.colour.append([float(j/255.0) for j in colour_int])
        f.close()
        
        self.vertex = np.array(self.vertex)
        self.normal = []
        for f in self.face:
            verts = [self.vertex[f[0]],self.vertex[f[1]],self.vertex[f[2]]]
            self.normal.append(np.cross(verts[1]-verts[0],verts[2]-verts[0]))
            self.normal[-1] = self.normal[-1]/np.linalg.norm(self.normal[-1])
        self.normal = np.array(self.normal)

def init(mainpath,screen_res):
    
    # Mission Data
    global all_mission_data
    all_mission_data = {}
    
    ###############
    # Mission test
    name = 'mission_test'
    all_mission_data[name] = {}
    all_mission_data[name]['music'] = 'into_the_unknown'
    all_mission_data[name]['player'] = [0.0,0.0,0.0]
    all_mission_data[name]['jump'] = [100.0,20.0,0.0]
    all_mission_data[name]['init_voice'] = None
    
    all_mission_data[name]['ships'] = [['interceptor',[300.0,0.0,0.0],'id1'], \
        ['interceptor',[300.0,20.0,0.0],'id2'],\
        ['transport',[50.0,0.0,0.0],'id3']]
    """
    all_mission_data[name]['ships'] = [['comm_sat',[0.0,0.0,0.0],'id1'],\
        ['comm_sat',[30.0,0.0,0.0],'id2'],\
        ['comm_sat',[0.0,40.0,0.0],'id3'],\
        ['comm_sat',[6.0,32.0,-8.0],'id4'],\
        ['defence_plat_a',[50.0,-20.0,-8.0],'id5'],\
        ['defence_plat_b',[-50.0,-20.0,8.0],'id6'],\
        ['defence_plat_a',[50.0,20.0,-8.0],'id7'] ]
    """
    
    all_mission_data[name]['obj'] = [['ship',[0],'Destroy ship'],\
        ['jump',0,'Proceed to Jump Gate']]
    
    ###############
    # Training Mission
    name = 'training'
    all_mission_data[name] = {}
    all_mission_data[name]['music'] = 'into_the_unknown'
    all_mission_data[name]['player'] = [0.0,0.0,0.0]
    all_mission_data[name]['jump'] = [200.0,0.0,-20.0]
    all_mission_data[name]['init_voice'] = 'train004'
    all_mission_data[name]['next_mission'] = 'mission1'
    
    all_mission_data[name]['ships'] = [['nav_beacon',[50.0,-20.0,0.0],'id1'],\
        ['nav_beacon',[50.0,20.0,0.0],'id2'],\
        ['nav_beacon',[100.0,-20.0,0.0],'id3'],\
        ['nav_beacon',[100.0,20.0,0.0],'id4'],\
        ['nav_beacon_green',[0.0,150.0,50.0],'id5']]
    
    all_mission_data[name]['obj'] = [['loc',[0.0,150.0,50.0],'Proceed to green navigation beacon (press o)','train005'], \
        ['ship',[4],'Destroy green navigation beacon','train006'], \
        ['jump',0,'Proceed to Jump Gate Alpha']]
    
    ########################################
    # Mission 1: Communications Satellites
    name = 'mission1'
    all_mission_data[name] = {}
    all_mission_data[name]['music'] = 'into_the_unknown'
    all_mission_data[name]['player'] = [0.0,0.0,0.0]
    all_mission_data[name]['jump'] = [650.0,-100.0,-150.0]
    all_mission_data[name]['init_voice'] = 'mission1a'
    
    all_mission_data[name]['ships'] = [['nav_beacon',[80.0,0.0,0.0],'id1'],\
        ['nav_beacon',[90.0,40.0,0.0],'id2'],\
        ['nav_beacon',[300.0,80.0,10.0],'id3'],\
        ['nav_beacon',[500.0,100.0,-5.0],'id4'],\
        ['comm_sat',[350.0,95.0,0.0],'id5'],\
        ['comm_sat',[450.0,120.0,-20.0],'id6'],\
        ['comm_sat',[600.0,130.0,-150.0],'id7'] ]
    
    all_mission_data[name]['obj'] = [['loc',[300.0,80.0,10.0],'Proceed to specified waypoint (press o)','mission1b'], \
        ['ship',[4,5,6],'Destroy three communications satellites','mission1c'], \
        ['jump',0,'Proceed to Jump Gate Lambda']]
    
    ########################################
    # Mission 2: Communications Node
    name = 'mission2'
    all_mission_data[name] = {}
    all_mission_data[name]['music'] = 'into_the_unknown'
    all_mission_data[name]['player'] = [0.0,0.0,0.0]
    all_mission_data[name]['jump'] = [40.0,-5.0,150.0]
    all_mission_data[name]['init_voice'] = 'mission2a'
    all_mission_data[name]['next_mission'] = 'mission3'
    
    all_mission_data[name]['ships'] = [['comm_sat',[300.0,10.0,0.0],'id1'],\
        ['comm_sat',[300.0,30.0,0.0],'id2'],\
        ['defence_plat_a',[300.0,20.0,0.0],'id3'], \
        ['comm_sat',[190.0,330.0,-100.0],'id4'],\
        ['comm_sat',[200.0,330.0,-100.0],'id5'],\
        ['defence_plat_a',[210.0,330.0,-100.0],'id6'], \
        ['comm_sat',[-100.0,70.0,-270.0],'id7'],\
        ['comm_sat',[-100.0,90.0,-270.0],'id8'],\
        ['defence_plat_a',[-100.0,80.0,-270.0],'id9']]
    
    all_mission_data[name]['obj'] = [['ship',[0,1,3,4,6,7],'Destroy six communications satellites.\nUse (e) to show hostile units.','mission2b'], \
        ['jump',0,'Proceed to Jump Gate Epsilon']]
    
    ########################################
    # Mission 3: Transport Attack
    name = 'mission3'
    all_mission_data[name] = {}
    all_mission_data[name]['music'] = 'into_the_unknown'
    all_mission_data[name]['player'] = [0.0,0.0,0.0]
    all_mission_data[name]['jump'] = [600.0,-300.0,-150.0]
    all_mission_data[name]['init_voice'] = 'mission3a'
    
    all_mission_data[name]['ships'] = [['defence_plat_b',[600.0,-110.0,200.0],'id1'], \
        ['defence_plat_b',[600.0,-70.0,200.0],'id3'], \
        ['interceptor',[500.0,-80.0,200.0],'id3'], \
        ['transport',[600.0,-90,200.0],'id4']]
    
    all_mission_data[name]['obj'] = [['ship',[3],'Destroy stranded security transport.','mission3b'], \
        ['jump',0,'Proceed to Jump Gate Beta']]
    
    ########################################
    # Mission 4: Civilian Transport Attack
    name = 'mission4'
    all_mission_data[name] = {}
    all_mission_data[name]['music'] = 'into_the_unknown'
    all_mission_data[name]['player'] = [0.0,0.0,0.0]
    all_mission_data[name]['jump'] = [700.0,300.0,50.0]
    all_mission_data[name]['init_voice'] = 'mission4a'
    
    all_mission_data[name]['ships'] = [['interceptor',[600.0,-10.0,5.0],'id5'], \
        ['interceptor',[250.0,10.0,5.0],'id6'], \
        ['interceptor',[350.0,10.0,5.0],'id7'], \
        ['interceptor',[500.0,10.0,5.0],'id8'], \
        ['transport',[600.0,0.0,0.0],'id9'], \
        ['transport',[600.0,5.0,20.0],'id10']]
    
    all_mission_data[name]['obj'] = [['ship',[4,5],'Destroy Earth transport shuttles.','mission4b'], \
        ['jump',0,'Proceed to Jump Gate Omega']]
    
    ########################################
    # Mission 5: Gate Attack
    name = 'mission5'
    all_mission_data[name] = {}
    all_mission_data[name]['music'] = 'into_the_unknown'
    all_mission_data[name]['player'] = [0.0,0.0,0.0]
    all_mission_data[name]['jump'] = [50.0,600.0,10.0]
    all_mission_data[name]['init_voice'] = 'mission5a'
    all_mission_data[name]['next_mission'] = 'mission6'
    
    all_mission_data[name]['ships'] = [['interceptor',[60.0,550.0,5.0],'id5'], \
        ['interceptor',[-10.0,590.0,5.0],'id6'], \
        ['defence_plat_b',[60.0,600.0,0.0],'id3']]
    
    all_mission_data[name]['obj'] = [['ship',[0,1,2],'Proceed to jump gate and eliminate defending forces.','mission5b'],\
        ['jump',0,'Proceed to Jump Gate']]
    
    ########################################
    # Mission 6: Earth Attack
    name = 'mission6'
    all_mission_data[name] = {}
    all_mission_data[name]['music'] = 'into_the_unknown'
    all_mission_data[name]['player'] = [0.0,0.0,0.0]
    all_mission_data[name]['jump'] = [100.0,550.0,10.0]
    all_mission_data[name]['init_voice'] = 'mission6a'
    
    all_mission_data[name]['ships'] = [['station',[60.0,550.0,5.0],'id1']]
    
    all_mission_data[name]['obj'] = [['ship',[0],'Eliminate civilian station in high Earth orbit.','mission6b'],\
        ['jump',0,'Proceed to Jump Gate']]
    
    # Models
    """
    global model_rock
    model_rock = Model()
    model_rock.Load(os.path.join(mainpath,'data','models','sphere_134.ply'))
    """
    
    global model_debris
    model_debris = Model()
    model_debris.Load(os.path.join(mainpath,'data','models','debris.ply'))
    
    global model_jumpgate
    model_jumpgate = Model()
    model_jumpgate.Load(os.path.join(mainpath,'data','models','jump_gate.ply'))
    
    global model_navbeacon
    model_navbeacon = Model()
    model_navbeacon.Load(os.path.join(mainpath,'data','models','nav_beacon.ply'))
    
    global model_navbeacongreen
    model_navbeacongreen = Model()
    model_navbeacongreen.Load(os.path.join(mainpath,'data','models','nav_beacon_green.ply'))
    
    global model_comm_sat
    model_comm_sat = Model()
    model_comm_sat.Load(os.path.join(mainpath,'data','models','comm_sat.ply'))
    
    global model_defence_plat_a
    model_defence_plat_a = Model()
    model_defence_plat_a.Load(os.path.join(mainpath,'data','models','defence_plat_a.ply'))
    
    global model_defence_plat_b
    model_defence_plat_b = Model()
    model_defence_plat_b.Load(os.path.join(mainpath,'data','models','defence_plat_b.ply'))
    
    global model_interceptor
    model_interceptor = Model()
    model_interceptor.Load(os.path.join(mainpath,'data','models','interceptor.ply'))
    
    global model_transport
    model_transport = Model()
    model_transport.Load(os.path.join(mainpath,'data','models','transport.ply'))
    
    global model_station
    model_station = Model()
    model_station.Load(os.path.join(mainpath,'data','models','station.ply'))
    
    # Lights
    global light_main
    light_main = model.Light([0.0,10.0,10.0],[1.0,1.0,1.0],type='distant')
    
    global light_main2
    light_main2 = model.Light([0.0,-10.0,0.0],[1.0,1.0,1.0],type='distant')
    
    #global light_pg1
    #light_pg1 = model.Light([0.0,0.0,10.0],[0.5,1.0,0.5],type='point',radius=[9.0,20.0])
    
    # Sun
    global img_sun
    img_sun = pygame.image.load(os.path.join(mainpath,'data','gfx','sun.png')).convert()
    img_sun.set_colorkey((255,0,255))
    
    # stars
    global img_stars
    img_stars = pygame.image.load(os.path.join(mainpath,'data','gfx','stars.png')).convert()
    img_stars.set_colorkey((255,0,255))
    
    # horizons
    global img_horizon_array
    img_horizon_array = {}
    
    img_horizon_clouds = pygame.image.load(os.path.join(mainpath,'data','gfx','sky_stars001.png')).convert()
    img_horizon_array['stars001'] = pygame.surfarray.array3d(img_horizon_clouds)
    img_horizon_clouds2 = pygame.image.load(os.path.join(mainpath,'data','gfx','sky_stars_earth.png')).convert()
    img_horizon_array['stars002'] = pygame.surfarray.array3d(img_horizon_clouds2)
    
    global img_horizon_surface
    img_horizon_surface = pygame.Surface((640, 128))
    img_horizon_surface.fill((255,0,255))
    img_horizon_surface.set_colorkey((255,0,255))
    
    # HUD Graphics
    global img_targets_sprite
    img_targets_sprite = pygame.image.load(os.path.join(mainpath,'data','gfx','targets.png')).convert()
    img_targets_sprite.set_colorkey((255,0,255))
    
    global coords_targets_sprite
    coords_targets_sprite = {}
    coords_targets_sprite['reticule'] = (0, 0, 32, 32)
    coords_targets_sprite['frame'] = {}
    coords_targets_sprite['frame']['enemy'] = (32, 0, 32, 32)
    coords_targets_sprite['frame']['objective'] = (64, 0, 32, 32)
    coords_targets_sprite['frame']['jump'] = (96, 0, 32, 32)
    coords_targets_sprite['arrow'] = {}
    coords_targets_sprite['arrow']['enemy'] = [(0, 32, 32, 32),(32, 32, 32, 32),(64, 32, 32, 32),(96, 32, 32, 32)]
    coords_targets_sprite['arrow']['objective'] = [(0, 64, 32, 32),(32, 64, 32, 32),(64, 64, 32, 32),(96, 64, 32, 32)]
    coords_targets_sprite['arrow']['green'] = [(0, 128, 32, 32),(32, 128, 32, 32),(64, 128, 32, 32),(96, 128, 32, 32)]
    
    coords_targets_sprite['mouse'] = (0, 96, 32, 32)
    
    # HUD status bar
    #fonthuddata = pygame.font.Font(os.path.join('data','KBZipaDeeDooDah.ttf'), 24)
    global font_huddata
    #font_huddata = pygame.font.Font(None, 18)
    font_huddata = pygame.font.Font(os.path.join(mainpath,'data','fonts','Silom.ttf'), 12)
    global font_huddata2
    #font_huddata2 = pygame.font.Font(None, 22)
    font_huddata2 = pygame.font.Font(os.path.join(mainpath,'data','fonts','Silom.ttf'), 20)
    
    global HUD_surfs
    HUD_surfs = {}
    
    HUD_surfs['shields'] = font_huddata.render("shields", True, (255, 255, 255), (0,0,0)).convert()
    HUD_surfs['shields'].set_colorkey((0,0,0))
    HUD_surfs['obj'] = font_huddata.render("Objective:", True, (255, 255, 255), (0,0,0)).convert()
    HUD_surfs['obj'].set_colorkey((0,0,0))
    
    HUD_surfs['failed'] = font_huddata2.render("Mission Failed", True, (210, 0, 0), (0,0,0)).convert()
    HUD_surfs['failed'].set_colorkey((0,0,0))
    
    HUD_surfs['jump'] = font_huddata2.render("Prepare for Hyperspace Jump", True, (210, 0, 0), (0,0,0)).convert()
    HUD_surfs['jump'].set_colorkey((0,0,0))
    
    # Objective descriptions
    HUD_surfs['objectives'] = {}
    for mission_name in all_mission_data.keys():
        HUD_surfs['objectives'][mission_name] = []
        for i in xrange(len(all_mission_data[mission_name]['obj'])):
            objective_str = all_mission_data[mission_name]['obj'][i][2]
            HUD_surfs['objectives'][mission_name].append(render_textrect(objective_str, font_huddata, pygame.Rect((0,0),(300,40)), (0, 204, 204), (0,0,0), 0).convert())
            #HUD_surfs['objectives'][mission_name].append(font_huddata.render(objective_str, True, (0, 204, 204), (0,0,0)).convert())
            HUD_surfs['objectives'][mission_name][-1].set_colorkey((0,0,0))
    
    # Sound Data
    global soundfx
    soundfx = {}
    
    soundfx['laser'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','laser.ogg'))
    soundfx['laser2'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','laser2.ogg'))
    soundfx['explode'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','explode.ogg'))
    soundfx['shields_low'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','shieldslow.ogg'))
    soundfx['jump'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','jump.ogg'))
    soundfx['jumpoffline'] = pygame.mixer.Sound(os.path.join(mainpath,'data','snd','jumpoffline.ogg'))
    
    # Voice Data
    global voicefx
    voicefx = {}
    
    voicefx['train001'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','train001.ogg'))
    voicefx['train002'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','train002.ogg'))
    voicefx['train003'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','train003.ogg'))
    voicefx['train004'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','train004.ogg'))
    voicefx['train005'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','train005.ogg'))
    voicefx['train006'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','train006.ogg'))
    
    voicefx['mission1a'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission1a.ogg'))
    voicefx['mission1b'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission1b.ogg'))
    voicefx['mission1c'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission1c.ogg'))
    
    voicefx['mission2a'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission2a.ogg'))
    voicefx['mission2b'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission2b.ogg'))
    
    voicefx['mission3a'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission3a.ogg'))
    voicefx['mission3b'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission3b.ogg'))
    
    voicefx['mission4a'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission4a.ogg'))
    voicefx['mission4b'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission4b.ogg'))
    
    voicefx['mission5a'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission5a.ogg'))
    voicefx['mission5b'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission5b.ogg'))
    
    voicefx['mission6a'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission6a.ogg'))
    voicefx['mission6b'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','mission6b.ogg'))
    
    voicefx['new_obj'] = pygame.mixer.Sound(os.path.join(mainpath,'data','voice','new_obj.ogg'))
    
    global training_voicelist
    training_voicelist = ['train001','train002','train003']
    
    # Assign sound to channels
    channels = []
    for i in xrange(7):
        channels.append(pygame.mixer.Channel(i+1))
    
	global next_sound_chan
	next_sound_chan = 0
	global PlaySound
	def PlaySound(name):
	    global next_sound_chan
	    channels[next_sound_chan].play(soundfx[name])
	    next_sound_chan += 1
	    if next_sound_chan == 7:
	        next_sound_chan = 0
	
	# Function for playing voice on dedicated channel
	voice_channel = pygame.mixer.Channel(0)
	global PlayVoice
	def PlayVoice(name):
	    voice_channel.play(voicefx[name])
    
    global PauseVoice
    def PauseVoice():
	    voice_channel.pause()
    
    global ResumeVoice
    def ResumeVoice():
	    voice_channel.unpause()
    
    global StopVoice
    def StopVoice():
	    voice_channel.stop()
	
    global CheckVoice
    def CheckVoice():
	    return voice_channel.get_busy()
	
    # Music Data
    global musicpaths
    musicpaths = {}
    musicpaths['title'] = os.path.join(mainpath,'data','music','title.ogg')
    musicpaths['into_the_unknown'] = os.path.join(mainpath,'data','music','into_the_unknown.ogg')
    
    # Control 
    
    # pre-sets and controls
    global controlmap
    controlmap = {}
    controlmap['R'] = K_d
    controlmap['L'] = K_a
    controlmap['D'] = K_s
    controlmap['U'] = K_w
    controlmap['Fire'] = K_SPACE
    
    controlmap['Hostile'] = K_e
    controlmap['Objective'] = K_o
    
    # Title screen data
    global titlescreen_data
    
    titlescreen_data = {}
    
    titlescreen_data['titlebg'] = pygame.image.load(os.path.join(mainpath,'data','gfx','title_background.png')).convert()
    titlescreen_data['titlebg'].set_colorkey((0,0,0))
    titlescreen_data['title_text'] = pygame.image.load(os.path.join(mainpath,'data','gfx','title_text.png')).convert()
    titlescreen_data['title_text'].set_colorkey((255,0,255))
    
    global font_titlescreen
    #font_titlescreen = pygame.font.Font(None, 22)
    font_titlescreen = pygame.font.Font(os.path.join(mainpath,'data','fonts','Silom.ttf'), 20)
    
    titlescreen_data['newgame_off'] = font_titlescreen.render("New Game", True, (0, 210, 0), (0,0,0)).convert()
    titlescreen_data['newgame_off'].set_colorkey((0,0,0))
    titlescreen_data['newgame_on'] = font_titlescreen.render("New Game", True, (210, 0, 0), (0,0,0)).convert()
    titlescreen_data['newgame_on'].set_colorkey((0,0,0))
    titlescreen_data['continue_off'] = font_titlescreen.render("Continue", True, (0, 210, 0), (0,0,0)).convert()
    titlescreen_data['continue_off'].set_colorkey((0,0,0))
    titlescreen_data['continue_on'] = font_titlescreen.render("Continue", True, (210, 0, 0), (0,0,0)).convert()
    titlescreen_data['continue_on'].set_colorkey((0,0,0))
    
    titlescreen_data['newgamepos'] = (450,340)
    titlescreen_data['continuepos'] = (450,380)
    
    # Menu Data
    global pausemenu_data
    
    pausemenu_data = {}
    
    pausemenu_data['continue_off'] = font_titlescreen.render("Continue Mission", True, (0, 210, 0), (0,0,0)).convert()
    pausemenu_data['continue_off'].set_colorkey((0,0,0))
    pausemenu_data['continue_on'] = font_titlescreen.render("Continue Mission", True, (210, 0, 0), (0,0,0)).convert()
    pausemenu_data['continue_on'].set_colorkey((0,0,0))
    pausemenu_data['quit_off'] = font_titlescreen.render("Quit Mission", True, (0, 210, 0), (0,0,0)).convert()
    pausemenu_data['quit_off'].set_colorkey((0,0,0))
    pausemenu_data['quit_on'] = font_titlescreen.render("Quit Mission", True, (210, 0, 0), (0,0,0)).convert()
    pausemenu_data['quit_on'].set_colorkey((0,0,0))
    pausemenu_data['reset_off'] = font_titlescreen.render("Reset Mission", True, (0, 210, 0), (0,0,0)).convert()
    pausemenu_data['reset_off'].set_colorkey((0,0,0))
    pausemenu_data['reset_on'] = font_titlescreen.render("Reset Mission", True, (210, 0, 0), (0,0,0)).convert()
    pausemenu_data['reset_on'].set_colorkey((0,0,0))
    
    pausemenu_data['adjustmouse'] = font_titlescreen.render("Adjust Mouse Sensitivity", True, (128, 128, 128), (0,0,0)).convert()
    pausemenu_data['adjustmouse'].set_colorkey((0,0,0))
    
    pausemenu_data['positive_on'] = pygame.Surface((32,32)).convert()
    pausemenu_data['positive_on'].fill((255,255,255))
    pausemenu_data['positive_on'].blit(img_targets_sprite,(0,0),area=coords_targets_sprite['arrow']['enemy'][2])
    pausemenu_data['positive_on'].set_colorkey((255,255,255))
    pausemenu_data['positive_off'] = pygame.Surface((32,32)).convert()
    pausemenu_data['positive_off'].fill((255,255,255))
    pausemenu_data['positive_off'].blit(img_targets_sprite,(0,0),area=coords_targets_sprite['arrow']['green'][2])
    pausemenu_data['positive_off'].set_colorkey((255,255,255))
    
    pausemenu_data['negative_on'] = pygame.Surface((32,32)).convert()
    pausemenu_data['negative_on'].fill((255,255,255))
    pausemenu_data['negative_on'].blit(img_targets_sprite,(0,0),area=coords_targets_sprite['arrow']['enemy'][0])
    pausemenu_data['negative_on'].set_colorkey((255,255,255))
    pausemenu_data['negative_off'] = pygame.Surface((32,32)).convert()
    pausemenu_data['negative_off'].fill((255,255,255))
    pausemenu_data['negative_off'].blit(img_targets_sprite,(0,0),area=coords_targets_sprite['arrow']['green'][0])
    pausemenu_data['negative_off'].set_colorkey((255,255,255))
    
    pausemenu_data['continuepos'] = (320-pausemenu_data['continue_off'].get_width()/2,80)
    pausemenu_data['quitpos'] = (320-pausemenu_data['quit_off'].get_width()/2,120)
    pausemenu_data['resetpos'] = (320-pausemenu_data['reset_off'].get_width()/2,160)
    
    pausemenu_data['adjustmousepos'] = (320-pausemenu_data['adjustmouse'].get_width()/2,300)
    pausemenu_data['positivepos'] = (440-16,330)
    pausemenu_data['negativepos'] = (200-16,330)
    
    # Cutscenes
    
    # cutscene text
    cutscene_text = {}
    
    cutscene_text['intro001'] = "Life on the colonies is all I've ever known."
    
    cutscene_text['intro002'] = "My parents were asteroid miners. My parent's parents were asteroid miners ... so it goes back pretty much as far as anyone can remember."
    cutscene_text['intro003'] = "Life's pretty straightforward out here. Scratch away at a piece of rock for eighteen hours of an Earth day, sleep when you can. Pray the government controlled prices for ore don't drop any lower than the measly crumbs they throw us now."
    cutscene_text['intro004'] = "That was my life and the life of my family ... until they came and took it all away."
    
    cutscene_text['intro005'] = "I had a daughter. A wife. A brother."
    cutscene_text['intro006'] = "I was down in the mines when the government warships bombed the colonies. Our habitat was wiped off the side of the asteroid. I never saw my wife and child again."
    cutscene_text['intro007'] = '"Colateral damage" is what they call it ... they blamed the insurgency, the "Brotherhood of Free Men" ... too many attacks on government security forces, so they pushed back.'
    cutscene_text['intro008'] = "Truth is they don't give a damn about offworlders. We're not even human to them."
    
    cutscene_text['intro009'] = "I fell into a downward spiral of drinking and despair. When I was at my lowest, I met a man who told me I could do something to make a difference, to hit back at those that taken away everything I had."
    cutscene_text['intro010'] = 'I went to work for him. He said the "Brotherhood" could make good use of the piloting skills I had built up operating haul craft in the mines.'
    cutscene_text['intro011'] = "He told me they had a special mission just right for man like me. Involves some spaceflight. If I can prove I'm a good pilot, I've got the job."
    cutscene_text['intro012'] = "All I know is that I've got nothing else to live for, and I'll make the rotten bastard government pay for what they've done."
    
    cutscene_text['mission2_001'] = 'So this is my "special mission", what I signed up for. Shooting rusty old satellites out of the sky. My every move dictated by a bunch of ones and zeros with the persona of a blank piece of paper ...'
    cutscene_text['mission2_002'] = "Why am I wasting my time here when I could be going after the real enemy?"
    
    cutscene_text['mission4_001'] = "When I saw that shuttle disintergrate into a thousand pieces, I thought I would feel something. Some sort of release, some feeling that justice had been served."
    cutscene_text['mission4_002'] = "The fighting takes the edge off the pain of grief, but when it's over, nothing's really changed."
    cutscene_text['mission4_003'] = "Those bastards got what they deserved, and I'll keep going until they're wiped off the colonies, but it doesn't change what I've already lost."
    
    cutscene_text['mission5_001'] = "In the days following the raid on those civilian transports, I had many uneasy thoughts on the path I had now taken."
    cutscene_text['mission5_002'] = "Wiping government security forces is one thing, but murdering their families too ... an eye for an eye? What had I become?"
    cutscene_text['mission5_003'] = "And if future operations required this sort of dirty work? What then?"
    
    cutscene_text['mission6_001'] = "Did hundreds of innocent lives deserve to die today, all in the name of a cause, an ideal? When presented with two choices, both of which are evil, which does one choose? The lesser?"
    cutscene_text['mission6_002'] = "I'm so tired of the fighting ..."
    
    # cutscene surface data
    global cutscenesurfs
    cutscenesurfs = {}
    
    font_cutscenes = pygame.font.Font(os.path.join(mainpath,'data','fonts','Silom.ttf'), 20)
    
    for k in cutscene_text.keys():
        cutscenesurfs[k] = render_textrect(cutscene_text[k], font_cutscenes, pygame.Rect((0,0),(600,200)), (0, 210, 0), (0,0,0), 1)
        cutscenesurfs[k].set_colorkey((0,0,0))
    
    # Backdrops
    cutscenesurfs['intro_backdrop1'] = pygame.image.load(os.path.join(mainpath,'data','gfx','asteroid_field_by_bull53y3.png')).convert()
    cutscenesurfs['intro_backdrop2'] = pygame.image.load(os.path.join(mainpath,'data','gfx','asteroid_stock.png')).convert()
    
    # Cutscene programming
    global introcutscene_data
    introcutscene_data = [[ ['none',['intro001']], \
        ['intro_backdrop1',['intro002','intro003','intro004']], \
        ['intro_backdrop2',['intro005','intro006','intro007','intro008']], \
        ['none',['intro009','intro010','intro011','intro012']] ], \
        'none','maingame',[True,'training']]
    
    global mission2cutscene_data
    mission2cutscene_data = [[ ['none',['mission2_001','mission2_002']] ], \
        'none','maingame',[True,'mission2']]
    all_mission_data['mission1']['next_mission'] = ['mission2cutscene',mission2cutscene_data]
    
    global mission4cutscene_data
    mission4cutscene_data = [[ ['none',['mission4_001','mission4_002','mission4_003']] ], \
        'none','maingame',[True,'mission4']]
    all_mission_data['mission3']['next_mission'] = ['mission4cutscene',mission4cutscene_data]
    
    global mission5cutscene_data
    mission5cutscene_data = [[ ['none',['mission5_001','mission5_002','mission5_003']] ], \
        'none','maingame',[True,'mission5']]
    all_mission_data['mission4']['next_mission'] = ['mission5cutscene',mission5cutscene_data]
    
    global mission6cutscene_data
    mission6cutscene_data = [[ ['none',['mission6_001','mission6_002']] ], \
        'title','titlescreen',[]]
    all_mission_data['mission6']['next_mission'] = ['mission6cutscene',mission6cutscene_data]
