#!/usr/bin/python

"""
visuals.py - functions to deal with visuals including sky, stars etc.
"""

import os, sys
import pygame
from pygame.locals import *
import numpy as np
from math import *
import random

import resources

class MotionDust(object):
    def __init__(self,parent,camera,N=50):
        self.parent = parent
        self.camera = camera
        self.gridsize = 15.0
        self.pos = self.gridsize*np.random.random((N,3))
        self.active = False
    
    def Update(self):
        indx = int(self.camera.pos[0]/self.gridsize)
        indy = int(self.camera.pos[1]/self.gridsize)
        indz = int(self.camera.pos[2]/self.gridsize)
        if (self.camera.pos[0] % self.gridsize) > (self.gridsize/2):
            dirx = 1
        else:
            dirx = -1
        if (self.camera.pos[1] % self.gridsize) > (self.gridsize/2):
            diry = 1
        else:
            diry = -1
        if (self.camera.pos[2] % self.gridsize) > (self.gridsize/2):
            dirz = 1
        else:
            dirz = -1
        new_pos = np.array([[10000.0,10000.0,10000.0],[10000.0,10000.0,10000.0]])
        for xi in [indx,indx+dirx]:
            for yi in [indy,indy+diry]:
                for zi in [indz,indz+dirz]:
                    new_pos = np.append(new_pos,self.pos+[self.gridsize*xi,self.gridsize*yi,self.gridsize*zi],axis=0)
        new_pos2 = new_pos + 1.0*np.array(self.parent.motion.velocity)
        
        self.vertex_c = new_pos-self.camera.pos
        self.range1 = np.linalg.norm(self.vertex_c,axis=1)
        self.depth1 = np.dot(self.vertex_c,self.camera.DCM[0])
        self.x1 = self.camera.f*np.dot(self.vertex_c,self.camera.DCM[1])/self.depth1 + self.camera.uv0[0]
        self.y1 = self.camera.f*np.dot(self.vertex_c,self.camera.DCM[2])/self.depth1 + self.camera.uv0[1]
        self.vertex_c = new_pos2-self.camera.pos
        self.depth2 = np.dot(self.vertex_c,self.camera.DCM[0])
        self.x2 = self.camera.f*np.dot(self.vertex_c,self.camera.DCM[1])/self.depth2 + self.camera.uv0[0]
        self.y2 = self.camera.f*np.dot(self.vertex_c,self.camera.DCM[2])/self.depth2 + self.camera.uv0[1]
        self.active = True

class StarField(object):
    def __init__(self,camera,N=100):
        self.camera = camera
        self.pos = []
        self.colour = []
        for i in xrange(N):
            r = 0.0
            while r == 0.0:
                pos = np.array([random.uniform(-1.0,1.0),random.uniform(-1.0,1.0),random.uniform(-1.0,1.0)])
                r = np.linalg.norm(pos)
            self.pos.append(pos/r)
            self.colour.append(random.randint(0,2))
            #brightness = random.uniform(0.1,1.0)
            #self.colour.append((255*brightness,255*brightness,255*brightness))
        self.pos = np.array(self.pos)
    
    def Draw(self,screen):
        depth = np.dot(self.pos,self.camera.DCM[0])
        x = self.camera.f*np.dot(self.pos,self.camera.DCM[1])/depth + self.camera.uv0[0]
        y = self.camera.f*np.dot(self.pos,self.camera.DCM[2])/depth + self.camera.uv0[1]
        for i in xrange(len(self.pos)):
            #screen.set_at((int(x[i]),int(y[i])), self.colour[i])
            #pygame.draw.circle(screen, self.colour[i], (int(x[i]),int(y[i])), 1, 0)
            coords = (self.colour[i]*3,0,3,3)
            screen.blit(resources.img_stars, (x[i], y[i]), area=coords)

class Sky(object):
    
    def __init__(self,camera,type='space'):
        self.camera = camera
        self.type = type
        
        self.sun_pos = [10.0,0.0,10.0]
        self.starfield = StarField(self.camera)
    
    def Draw(self,screen):
        
        # draw star field
        self.starfield.Draw(screen)
        
        # Draw Sun
        sundepth = np.dot(self.sun_pos,self.camera.DCM[0])
        xsun = self.camera.f*np.dot(self.sun_pos,self.camera.DCM[1])/sundepth + self.camera.uv0[0]
        ysun = self.camera.f*np.dot(self.sun_pos,self.camera.DCM[2])/sundepth + self.camera.uv0[1]
        if sundepth < 0.0 and xsun >= 0 and xsun < 640 and ysun >=0 and ysun < 480:
            screen.blit(resources.img_sun, (xsun-32, ysun-32))
        
        # Draw horizon
        horizon_y = self.camera.f*tan(asin(-self.camera.DCM[0][2]))
        heading = atan2(self.camera.DCM[0][1],self.camera.DCM[0][0]);
        
        anglerange = (1.0/0.785398)*(64.0/320)
        inds1 = (320*(np.arctan(np.arange(-anglerange,anglerange,anglerange/64)+(horizon_y/320))-(horizon_y/320))+32).astype(int)
        np.place(inds1, inds1<0, 64)
        np.place(inds1, inds1>63, 64)
        inds1a = np.tile(inds1,(640,1))
        
        #inds2 = (320*(np.arctan(np.arange(-320.0/320,320.0/320,1.0/320))+4*(heading/pi))+1280).astype(int)
        #anglerange = 0.785398
        anglerange = 1.0/0.785398
        #anglerange = 1.0
        inds2 = (320*(np.arctan(np.arange(-anglerange,anglerange,anglerange/320))+4*(heading/pi))+1280).astype(int)
        inds2 = np.remainder(inds2,2560)
        inds2a = np.tile(inds2,(128,1))
        
        image_array = resources.img_horizon_array[self.type][inds2a.T,inds1a]
        pygame.surfarray.blit_array(resources.img_horizon_surface, image_array)
        screen.blit(resources.img_horizon_surface, (0, horizon_y+240-48))

class ExplosionEffects(object):
    
    def __init__(self,parent,camera,Ne=5,Ns=5):
        self.camera = camera
        self.parent = parent
        self.explosions = []
        for i in xrange(Ne):
            self.explosions.append([0,0,0])
        self.sparks = []
        for i in xrange(Ns):
            self.sparks.append([0,0,0])
    
    def NewSpark(self,pos,to=10):
        for i in xrange(len(self.sparks)):
            if self.sparks[i][0] == 0:
                self.sparks[i] = [to,1,pos]
                break
    
    def NewExplosion(self,pos,vel,to=10,big=False):
        for i in xrange(len(self.explosions)):
            if self.explosions[i][0] == 0:
                if big:
                    self.explosions[i] = [2*to,1,pos]
                    for i in xrange(15):
                        v1 = 0.5*np.array([2*random.random()-1,2*random.random()-1,2*random.random()-1])
                        self.parent.debris.AddDebris(pos,vel+v1)
                else:
                    self.explosions[i] = [to,1,pos]
                    v1 = 0.5*np.array([random.random(),random.random(),random.random()])
                    v2 = 0.5*np.array([random.random(),random.random(),random.random()])
                    v3 = -0.5*(v1+v2)
                    self.parent.debris.AddDebris(pos,vel+v1)
                    self.parent.debris.AddDebris(pos,vel+v2)
                    self.parent.debris.AddDebris(pos,vel+v3)
                break
    
    def Update(self):
        for s in self.sparks:
            if s[0] > 0:
                s[1] += 1
                if s[1] >= s[0]:
                    s[0] = 0
        for e in self.explosions:
            if e[0] > 0:
                e[1] += 1
                if e[1] >= e[0]:
                    e[0] = 0
    
    def Draw(self,screen):
        for s in self.sparks:
            if s[0] > 0:
                radius = 2*s[1]
                vertex_c = s[2]-self.camera.pos
                depth = np.dot(vertex_c,self.camera.DCM[0])
                x = self.camera.f*np.dot(vertex_c,self.camera.DCM[1])/depth + self.camera.uv0[0]
                y = self.camera.f*np.dot(vertex_c,self.camera.DCM[2])/depth + self.camera.uv0[1]
                pygame.draw.circle(screen, (255,200,0), (int(x),int(y)), int(radius), 1)
        for e in self.explosions:
            if e[0] > 0:
                radius = 5*e[1]
                vertex_c = e[2]-self.camera.pos
                depth = np.dot(vertex_c,self.camera.DCM[0])
                x = self.camera.f*np.dot(vertex_c,self.camera.DCM[1])/depth + self.camera.uv0[0]
                y = self.camera.f*np.dot(vertex_c,self.camera.DCM[2])/depth + self.camera.uv0[1]
                pygame.draw.circle(screen, (255,200,0), (int(x),int(y)), int(radius), 1)
